@@ -31,15 +31,20 @@ int selfpipe[2];
 int sigc =0;
 int sigi =0;
 
-void sig_cont_handler (void) {
+void sig_cont_handler (int unused) {
   sigc++;
   write(selfpipe[1], "", 1);
 }
-void sig_int_handler (void) {
+void sig_int_handler (int unused) {
   sigi++;
   write(selfpipe[1], "", 1);
 }
-void sig_child_handler (void) { write(selfpipe[1], "", 1); }
+void sig_child_handler (int unused) { write(selfpipe[1], "", 1); }
+
+void sync_if_needed() {
+  struct stat s;
+  if (stat(NOSYNC, &s) == -1) sync();
+}
 
 int main (int argc, const char * const *argv, char * const *envp) {
   const char * prog[2];
@@ -49,6 +54,7 @@ int main (int argc, const char * const *argv, char * const *envp) {
   iopause_fd x;
 #ifndef IOPAUSE_POLL
   fd_set rfds;
+  struct timeval t;
 #endif
   char ch;
   int ttyfd;
@@ -89,7 +95,7 @@ int main (int argc, const char * const *argv, char * const *envp) {
   if (RB_DISABLE_CAD == 0) reboot_system(0);
 #endif
 
-  strerr_warn3(INFO, "$Id: runit.c,v 1.14 2006/11/21 15:09:18 pape Exp $",
+  strerr_warn3(INFO, "$Id: c66c1e3b98d1b630bd51b1d56aaed5c2994e8c9d $",
                ": booting.", 0);
 
   /* runit */
@@ -124,7 +130,7 @@ int main (int argc, const char * const *argv, char * const *envp) {
       sig_unblock(sig_child);
       sig_uncatch(sig_child);
       sig_unblock(sig_cont);
-      sig_ignore(sig_cont);
+      sig_uncatch(sig_cont);
       sig_unblock(sig_hangup);
       sig_unblock(sig_int);
       sig_uncatch(sig_int);
@@ -133,15 +139,11 @@ int main (int argc, const char * const *argv, char * const *envp) {
             
       strerr_warn3(INFO, "enter stage: ", stage[st], 0);
       execve(*prog, (char *const *)prog, envp);
-      strerr_die3sys(0, FATAL, "could not start child: ", stage[st]);
+      strerr_die4sys(0, FATAL, "unable to start child: ", stage[st], ": ");
     }
 
     x.fd =selfpipe[0];
     x.events =IOPAUSE_READ;
-#ifndef IOPAUSE_POLL
-    FD_ZERO(&rfds);
-    FD_SET(x.fd, &rfds);
-#endif
     for (;;) {
       int child;
 
@@ -149,16 +151,24 @@ int main (int argc, const char * const *argv, char * const *envp) {
       sig_unblock(sig_cont);
       sig_unblock(sig_int);
 #ifdef IOPAUSE_POLL
-      poll(&x, 1, -1);
+      poll(&x, 1, 14000);
 #else
-      select(x.fd +1, &rfds, (fd_set*)0, (fd_set*)0, (struct timeval*)0);
+      t.tv_sec =14; t.tv_usec =0;
+      FD_ZERO(&rfds);
+      FD_SET(x.fd, &rfds);
+      select(x.fd +1, &rfds, (fd_set*)0, (fd_set*)0, &t);
 #endif
       sig_block(sig_cont);
       sig_block(sig_child);
       sig_block(sig_int);
       
-      read(selfpipe[0], &ch, 1);
-      child =wait_nohang(&wstat);
+      while (read(selfpipe[0], &ch, 1) == 1) {}
+      while ((child =wait_nohang(&wstat)) > 0)
+        if (child == pid) break;
+      if (child == -1) {
+        strerr_warn2(WARNING, "wait_nohang, pausing: ", &strerr_sys);
+        sleep(5);
+      }
 
       /* reget stderr */
       if ((ttyfd =open_write("/dev/console")) != -1) {
@@ -167,7 +177,7 @@ int main (int argc, const char * const *argv, char * const *envp) {
       }
 
       if (child == pid) {
-        if (wait_exitcode(wstat) != 0) {
+        if (wait_crashed(wstat) || (wait_exitcode(wstat) != 0)) {
           if (wait_crashed(wstat))
             strerr_warn3(WARNING, "child crashed: ", stage[st], 0);
           else
@@ -194,7 +204,7 @@ int main (int argc, const char * const *argv, char * const *envp) {
         strerr_warn3(INFO, "leave stage: ", stage[st], 0);
         break;
       }
-      if (child > 0) {
+      if (child != 0) {
         /* collect terminated children */
         write(selfpipe[1], "", 1);
         continue;
@@ -224,7 +234,7 @@ int main (int argc, const char * const *argv, char * const *envp) {
           /* child */
           strerr_warn3(INFO, "enter stage: ", prog[0], 0);
           execve(*prog, (char *const *) prog, envp);
-          strerr_die3sys(0, FATAL, "could not start child: ", prog[0]);
+          strerr_die4sys(0, FATAL, "unable to start child: ", prog[0], ": ");
         }
         if (wait_pid(&wstat, pid2) == -1)
           strerr_warn2(FATAL, "wait_pid: ", &strerr_sys);
@@ -300,28 +310,28 @@ int main (int argc, const char * const *argv, char * const *envp) {
   case -1:
   if ((stat(REBOOT, &s) != -1) && (s.st_mode & S_IXUSR)) {
     strerr_warn2(INFO, "system reboot.", 0);
-    sync();
+    sync_if_needed();
     reboot_system(RB_AUTOBOOT);
   }
   else {
 #ifdef RB_POWER_OFF
     strerr_warn2(INFO, "power off...", 0);
-    sync();
+    sync_if_needed();
     reboot_system(RB_POWER_OFF);
     sleep(2);
 #endif
 #ifdef RB_HALT_SYSTEM
     strerr_warn2(INFO, "system halt.", 0);
-    sync();
+    sync_if_needed();
     reboot_system(RB_HALT_SYSTEM);
 #else
 #ifdef RB_HALT
     strerr_warn2(INFO, "system halt.", 0);
-    sync();
+    sync_if_needed();
     reboot_system(RB_HALT);
 #else
     strerr_warn2(INFO, "system reboot.", 0);
-    sync();
+    sync_if_needed();
     reboot_system(RB_AUTOBOOT);
 #endif
 #endif
