@@ -17,7 +17,7 @@
 #include "ndelay.h"
 
 #define USAGE " [-P] dir"
-#define VERSION "$Id: ecebd0a50510e91639c6a45dda8b0947aa8eb885 $"
+#define VERSION "$Id: 3ef5711e304a59f2004bf4922dc1848ef5249143 $"
 
 #define MAXSERVICES 1000
 
@@ -51,8 +51,8 @@ void warn(char *m1, char *m2) {
 void warn3x(char *m1, char *m2, char *m3) {
   strerr_warn6("runsvdir ", svdir, ": warning: ", m1, m2, m3, 0);
 } 
-void s_term() { exitsoon =1; }
-void s_hangup() { exitsoon =2; }
+void s_term(int unused) { exitsoon =1; }
+void s_hangup(int unused) { exitsoon =2; }
 
 void runsv(int no, char *name) {
   int pid;
@@ -63,7 +63,7 @@ void runsv(int no, char *name) {
   }
   if (pid == 0) {
     /* child */
-    const char *prog[3];
+    char *prog[3];
 
     prog[0] ="runsv";
     prog[1] =name;
@@ -71,7 +71,7 @@ void runsv(int no, char *name) {
     sig_uncatch(sig_hangup);
     sig_uncatch(sig_term);
     if (pgrp) setsid();
-    pathexec_run(*prog, prog, (const char* const*)environ);
+    pathexec_run(*prog, prog, (char* const*)environ);
     fatal("unable to start runsv ", name);
   }
   sv[no].pid =pid;
