@@ -34,7 +34,7 @@
 #include "iopause.h"
 
 #define USAGE " [-ttv] [-r c] [-R abc] [-l len] [-b buflen] dir ..."
-#define VERSION "$Id: 66d40067fb843ee6cde160e47c9dee5c73986f6d $"
+#define VERSION "$Id: 75895b2a099dd6ab02e9bcf480ae86011f9fc234 $"
 
 #define FATAL "svlogd: fatal: "
 #define WARNING "svlogd: warning: "
@@ -235,8 +235,11 @@ void rmoldest(struct logdir *ld) {
           warn2("unable to unlink processor leftover", f->d_name);
       }
       else {
-        ++n;
-        if (str_diff(f->d_name, oldest) < 0) byte_copy(oldest, 27, f->d_name);
+        ++n; /* count .s and .u files ... */
+        if (str_diff(f->d_name, oldest) < 0)
+          /* ... but don't unlink our pending .u if the clock is funny */
+          if ((ld->fnsave[26] != 'u') || str_diff(ld->fnsave, f->d_name))
+            byte_copy(oldest, 27, f->d_name);
       }
       errno =0;
     }
@@ -616,11 +619,11 @@ int buffer_pread(int fd, char *s, unsigned int len) {
   if (i > 0) linecomplete =(s[i -1] == '\n');
   return(i);
 }
-void sig_term_handler(void) {
+void sig_term_handler(int unused) {
   if (verbose) strerr_warn2(INFO, "sigterm received.", 0);
   exitasap =1;
 }
-void sig_child_handler(void) {
+void sig_child_handler(int unused) {
   int pid, l;
 
   if (verbose) strerr_warn2(INFO, "sigchild received.", 0);
@@ -632,11 +635,11 @@ void sig_child_handler(void) {
         break;
       }
 }
-void sig_alarm_handler(void) {
+void sig_alarm_handler(int unused) {
   if (verbose) strerr_warn2(INFO, "sigalarm received.", 0);
   rotateasap =1;
 }
-void sig_hangup_handler(void) {
+void sig_hangup_handler(int unused) {
   if (verbose) strerr_warn2(INFO, "sighangup received.", 0);
   reopenasap =1;
 }
